@@ -1,11 +1,12 @@
 /*
  * pam_auth.c -- PAM authentication
  *
- * $Id: pam_auth.c,v 1.4 2003/07/13 20:01:44 vorlon Exp $
+ * $Id: pam_auth.c,v 1.5 2005/03/29 20:41:20 toady Exp $
  *
  */
 
 #include "pam_private.h"
+#include "pam_prelude.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +41,10 @@ int pam_authenticate(pam_handle_t *pamh, int flags)
 	D(("will resume when ready"));
     }
 
+#ifdef PRELUDE
+    prelude_send_alert(pamh, retval);
+#endif
+
     return retval;
 }
 
