@@ -34,6 +34,7 @@
 #define PAM_SM_ACCOUNT
 
 #include <security/pam_modules.h>
+#include <security/_pam_modutil.h>
 
 /* some syslogging */
 
@@ -97,6 +98,15 @@ static int securetty_perform_check(pam_handle_t *pamh, int flags, int ctrl,
 	return (retval == PAM_CONV_AGAIN ? PAM_INCOMPLETE:PAM_SERVICE_ERR);
     }
 
+    user_pwd = _pammodutil_getpwnam(pamh, username);
+    if (user_pwd == NULL) {
+	return PAM_IGNORE;
+    } else if (user_pwd->pw_uid != 0) { /* If the user is not root,
+					   securetty's does not apply
+					   to them */
+	return PAM_SUCCESS;
+    }
+
     retval = pam_get_item(pamh, PAM_TTY, (const void **)&uttyname);
     if (retval != PAM_SUCCESS || uttyname == NULL) {
         if (ctrl & PAM_DEBUG_ARG) {
@@ -110,15 +120,6 @@ static int securetty_perform_check(pam_handle_t *pamh, int flags, int ctrl,
 	uttyname += sizeof(TTY_PREFIX)-1;
     }
 
-    user_pwd = getpwnam(username);
-    if (user_pwd == NULL) {
-	return PAM_IGNORE;
-    } else if (user_pwd->pw_uid != 0) { /* If the user is not root,
-					   securetty's does not apply
-					   to them */
-	return PAM_SUCCESS;
-    }
-
     if (stat(SECURETTY_FILE, &ttyfileinfo)) {
 	_pam_log(LOG_NOTICE, "Couldn't open " SECURETTY_FILE);
 	return PAM_SUCCESS; /* for compatibility with old securetty handling,
@@ -146,10 +147,10 @@ static int securetty_perform_check(pam_handle_t *pamh, int flags, int ctrl,
     } else {
 	ptname[0] = '\0';
     }
-    
+
     retval = 1;
 
-    while ((fgets(ttyfileline, sizeof(ttyfileline)-1, ttyfile) != NULL) 
+    while ((fgets(ttyfileline, sizeof(ttyfileline)-1, ttyfile) != NULL)
 	   && retval) {
 	if (ttyfileline[strlen(ttyfileline) - 1] == '\n')
 	    ttyfileline[strlen(ttyfileline) - 1] = '\0';
@@ -160,12 +161,10 @@ static int securetty_perform_check(pam_handle_t *pamh, int flags, int ctrl,
     fclose(ttyfile);
 
     if (retval) {
-	if (ctrl & PAM_DEBUG_ARG) {
 	    _pam_log(LOG_WARNING, "access denied: tty '%s' is not secure !",
 		     uttyname);
-	}
-	retval = PAM_AUTH_ERR;
 
+	    retval = PAM_AUTH_ERR;
     } else {
 	if ((retval == PAM_SUCCESS) && (ctrl & PAM_DEBUG_ARG)) {
 	    _pam_log(LOG_DEBUG, "access allowed for '%s' on '%s'",
