@@ -43,6 +43,7 @@
 #define PAM_SM_ACCOUNT
 
 #include <security/pam_modules.h>
+#include <security/_pam_modutil.h>
 
 /* some syslogging */
 
@@ -74,7 +75,8 @@ static int is_on_list(char * const *list, const char *member)
 #define PAM_DEBUG_ARG       0x0001
 #define PAM_USE_UID_ARG     0x0002
 #define PAM_TRUST_ARG       0x0004
-#define PAM_DENY_ARG        0x0010  
+#define PAM_DENY_ARG        0x0010
+#define PAM_ROOT_ONLY_ARG   0x0020
 
 static int _pam_parse(int argc, const char **argv, char *use_group,
 		      size_t group_length)
@@ -96,6 +98,8 @@ static int _pam_parse(int argc, const char **argv, char *use_group,
                ctrl |= PAM_TRUST_ARG;
           else if (!strcmp(*argv,"deny"))
                ctrl |= PAM_DENY_ARG;
+          else if (!strcmp(*argv,"root_only"))
+               ctrl |= PAM_ROOT_ONLY_ARG;
           else if (!strncmp(*argv,"group=",6))
 	       strncpy(use_group,*argv+6,group_length-1);
           else {
@@ -110,8 +114,8 @@ static int perform_check(pam_handle_t *pamh, int flags, int ctrl,
 			 const char *use_group)
 {
     const char *username = NULL;
-    char *fromsu;
-    struct passwd *pwd, *tpwd;
+    const char *fromsu;
+    struct passwd *pwd, *tpwd = NULL;
     struct group *grp;
     int retval = PAM_AUTH_ERR;
 
@@ -123,17 +127,22 @@ static int perform_check(pam_handle_t *pamh, int flags, int ctrl,
         return PAM_SERVICE_ERR;
     }
 
-    /* su to a uid 0 account ? */
-    pwd = getpwnam(username);
+    pwd = _pammodutil_getpwnam (pamh, username);
     if (!pwd) {
         if (ctrl & PAM_DEBUG_ARG) {
             _pam_log(LOG_NOTICE,"unknown user %s",username);
-	}
+        }
         return PAM_USER_UNKNOWN;
     }
+    if (ctrl & PAM_ROOT_ONLY_ARG) {
+	/* su to a non uid 0 account ? */
+        if (pwd->pw_uid != 0) {
+            return PAM_IGNORE;
+        }
+    }
      
     if (ctrl & PAM_USE_UID_ARG) {
-	tpwd = getpwuid(getuid());
+	tpwd = _pammodutil_getpwuid (pamh, getuid());
 	if (!tpwd) {
 	    if (ctrl & PAM_DEBUG_ARG) {
                 _pam_log(LOG_NOTICE, "who is running me ?!");
@@ -142,9 +151,9 @@ static int perform_check(pam_handle_t *pamh, int flags, int ctrl,
 	}
 	fromsu = tpwd->pw_name;
     } else {
-	fromsu = getlogin();
+	fromsu = _pammodutil_getlogin(pamh);
 	if (fromsu) {
-	    tpwd = getpwnam(fromsu);
+	    tpwd = _pammodutil_getpwnam (pamh, fromsu);
 	}
 	if (!fromsu || !tpwd) {
 	    if (ctrl & PAM_DEBUG_ARG) {
@@ -159,11 +168,11 @@ static int perform_check(pam_handle_t *pamh, int flags, int ctrl,
      */
      
     if (!use_group[0]) {
-	if ((grp = getgrnam("wheel")) == NULL) {
-	    grp = getgrgid(0);
+	if ((grp = _pammodutil_getgrnam (pamh, "wheel")) == NULL) {
+	    grp = _pammodutil_getgrgid (pamh, 0);
 	}
     } else {
-	grp = getgrnam(use_group);
+	grp = _pammodutil_getgrnam (pamh, use_group);
     }
 
     if (!grp || (!grp->gr_mem && (tpwd->pw_gid != grp->gr_gid))) {
